@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2011 Yoran Heling
+  Copyright (c) 2007-2012 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -31,6 +31,7 @@
 
 
 static int graph = 1, show_as = 0, info_show = 0, info_page = 0, info_start = 0;
+static char *message = NULL;
 
 
 
@@ -152,7 +153,7 @@ static void browse_draw_item(struct dir *n, int row) {
 void browse_draw() {
   struct dir *t;
   char fmtsize[9], *tmp;
-  int selected, i;
+  int selected = 0, i;
 
   erase();
   t = dirlist_get(0);
@@ -162,7 +163,9 @@ void browse_draw() {
   mvhline(0, 0, ' ', wincols);
   mvhline(winrows-1, 0, ' ', wincols);
   mvprintw(0,0,"%s %s ~ Use the arrow keys to navigate, press ? for help", PACKAGE_NAME, PACKAGE_VERSION);
-  if(read_only)
+  if(dir_import_active)
+    mvaddstr(0, wincols-10, "[imported]");
+  else if(read_only)
     mvaddstr(0, wincols-11, "[read-only]");
   attroff(A_REVERSE);
 
@@ -200,9 +203,16 @@ void browse_draw() {
       selected = i;
   }
 
+  /* draw message window */
+  if(message) {
+    nccreate(6, 60, "Message");
+    ncaddstr(2, 2, message);
+    ncaddstr(4, 34, "Press any key to continue");
+  }
+
   /* draw information window */
   t = dirlist_get(0);
-  if(info_show && t != dirlist_parent)
+  if(!message && info_show && t != dirlist_parent)
     browse_draw_info(t);
 
   /* move cursor to selected row for accessibility */
@@ -214,6 +224,12 @@ int browse_key(int ch) {
   struct dir *t, *sel;
   int i, catch = 0;
 
+  /* message window overwrites all keys */
+  if(message) {
+    message = NULL;
+    return 0;
+  }
+
   sel = dirlist_get(0);
 
   /* info window overwrites a few keys */
@@ -345,8 +361,15 @@ int browse_key(int ch) {
 
     /* and other stuff */
     case 'r':
-      if(sel != NULL)
-        calc_init(getpath(sel->parent), sel->parent);
+      if(dir_import_active) {
+        message = "Directory imported from file, won't refresh.";
+        break;
+      }
+      if(sel != NULL) {
+        dir_ui = 2;
+        dir_mem_init(sel->parent);
+        dir_scan_init(getpath(sel->parent));
+      }
       info_show = 0;
       break;
     case 'q':
@@ -368,7 +391,13 @@ int browse_key(int ch) {
       info_show = 0;
       break;
     case 'd':
-      if(read_only || sel == NULL || sel == dirlist_parent)
+      if(read_only || dir_import_active) {
+        message = read_only
+          ? "File deletion disabled in read-only mode."
+          : "File deletion not available for imported directories.";
+        break;
+      }
+      if(sel == NULL || sel == dirlist_parent)
         break;
       info_show = 0;
       if((t = dirlist_get(1)) == sel)
@@ -391,6 +420,7 @@ int browse_key(int ch) {
 
 void browse_init(struct dir *cur) {
   pstate = ST_BROWSE;
+  message = NULL;
   dirlist_open(cur);
 }
 
