@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2011 Yoran Heling
+  Copyright (c) 2007-2012 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -30,8 +30,8 @@
 
 /* public variables */
 struct dir *dirlist_parent = NULL;
-off_t  dirlist_maxs        = 0,
-       dirlist_maxa        = 0;
+int64_t dirlist_maxs       = 0,
+        dirlist_maxa       = 0;
 
 int    dirlist_sort_desc   = 1,
        dirlist_sort_col    = DL_COL_SIZE,
