@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2011 Yoran Heling
+  Copyright (c) 2007-2012 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -74,7 +74,7 @@ extern int dirlist_sort_desc, dirlist_sort_col, dirlist_sort_df;
 extern int dirlist_hidden;
 
 /* maximum size of an item in the opened dir */
-extern off_t dirlist_maxs, dirlist_maxa;
+extern int64_t dirlist_maxs, dirlist_maxa;
 
 
 #endif
