@@ -1,6 +1,6 @@
-/* ncdu - NCurses Disk Usage 
-    
-  Copyright (c) 2007-2011 Yoran Heling
+/* ncdu - NCurses Disk Usage
+
+  Copyright (c) 2007-2012 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -9,10 +9,10 @@
   distribute, sublicense, and/or sell copies of the Software, and to
   permit persons to whom the Software is furnished to do so, subject to
   the following conditions:
-  
+
   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.
-  
+
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -28,9 +28,17 @@
 
 #include "config.h"
 #include <stdio.h>
+#include <stddef.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
+
 /* File Flags (struct dir -> flags) */
 #define FF_DIR    0x01
 #define FF_FILE   0x02
@@ -48,25 +56,27 @@
 #define ST_HELP   3
 
 
-/* structure representing a file or directory
- * XXX: probably a good idea to get rid of the custom _t types and use
- *      fixed-size integers instead, which are much more predictable */
+/* structure representing a file or directory */
 struct dir {
+  int64_t size, asize;
+  uint64_t ino, dev;
   struct dir *parent, *next, *prev, *sub, *hlnk;
-  off_t size, asize;
-  ino_t ino;
-  unsigned long items;
-  dev_t dev;
+  int items;
   unsigned char flags;
   char name[3]; /* must be large enough to hold ".." */
-}; 
-/* sizeof(total dir) = SDIRSIZE + strlen(name) = sizeof(struct dir) - 3 + strlen(name) + 1 */
-#define SDIRSIZE (sizeof(struct dir)-2)
+};
+/* sizeof(total dir) = SDIRSIZE + strlen(name) = offsetof(struct dir, name) + strlen(name) + 1 */
+#define SDIRSIZE (offsetof(struct dir, name)+1)
 
-/* Ideally, the name array should be as large as the padding added to the end of
- * the struct, but I can't figure out a portable way to calculate this. We can
- * be sure that it's at least 3 bytes, though, as the struct is aligned to at
- * least 4 bytes and the flags field is a single byte. */
+/* A note on the ino and dev fields above: ino is usually represented as ino_t,
+ * which POSIX specifies to be an unsigned integer.  dev is usually represented
+ * as dev_t, which may be either a signed or unsigned integer, and in practice
+ * both are used. dev represents an index / identifier of a device or
+ * filesystem, and I'm unsure whether a negative value has any meaning in that
+ * context. Hence my choice of using an unsigned integer. Negative values, if
+ * we encounter them, will just get typecasted into a positive value. No
+ * information is lost in this conversion, and the semantics remain the same.
+ */
 
 
 /* program state */
@@ -81,13 +91,13 @@ int input_handle(int);
 
 
 /* import all other global functions and variables */
-#include "exclude.h"
-#include "util.h"
-#include "calc.h"
-#include "delete.h"
 #include "browser.h"
+#include "delete.h"
+#include "dir.h"
+#include "dirlist.h"
+#include "exclude.h"
 #include "help.h"
 #include "path.h"
-#include "dirlist.h"
+#include "util.h"
 
 #endif
