@@ -1,6 +1,6 @@
-/* ncdu - NCurses Disk Usage 
-    
-  Copyright (c) 2007-2011 Yoran Heling
+/* ncdu - NCurses Disk Usage
+
+  Copyright (c) 2007-2012 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -9,10 +9,10 @@
   distribute, sublicense, and/or sell copies of the Software, and to
   permit persons to whom the Software is furnished to do so, subject to
   the following conditions:
-  
+
   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.
-  
+
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -54,9 +54,9 @@ char *cropstr(const char *from, int s) {
 }
 
 
-char *formatsize(const off_t from) {
+char *formatsize(int64_t from) {
   static char dat[9]; /* "xxx.xMiB" */
-  float r = from; 
+  float r = from;
   char c = ' ';
   if(r < 1000.0f)      { }
   else if(r < 1023e3f) { c = 'K'; r/=1024.0f; }
@@ -68,10 +68,10 @@ char *formatsize(const off_t from) {
 }
 
 
-char *fullsize(const off_t from) {
+char *fullsize(int64_t from) {
   static char dat[20]; /* max: 999.999.999.999.999 */
   char tmp[20];
-  off_t n = from;
+  int64_t n = from;
   int i, j;
 
   /* the K&R method - more portable than sprintf with %lld */
@@ -111,7 +111,7 @@ int ncresize(int minrows, int mincols) {
       erase();
       refresh();
       endwin();
-      exit(0); 
+      exit(0);
     }
     if(ch == 'i')
       return 1;
@@ -121,7 +121,7 @@ int ncresize(int minrows, int mincols) {
 }
 
 
-void nccreate(int height, int width, char *title) {
+void nccreate(int height, int width, const char *title) {
   int i;
 
   subwinr = winrows/2-height/2;
@@ -175,7 +175,7 @@ static void freedir_hlnk(struct dir *d) {
    * This works the same as with adding: only the parents in which THIS is the
    * only occurence of the hard link will be modified, if the same file still
    * exists within the parent it shouldn't get removed from the count.
-   * XXX: Same note as for calc.c / calc_hlnk_check():
+   * XXX: Same note as for dir_mem.c / hlink_check():
    *      this is probably not the most efficient algorithm */
   for(i=1,par=d->parent; i&&par; par=par->parent) {
     if(d->hlnk)
@@ -212,12 +212,13 @@ static void freedir_rec(struct dir *dr) {
 
 
 void freedir(struct dir *dr) {
-  struct dir *tmp;
+  if(!dr)
+    return;
 
   /* free dr->sub recursively */
   if(dr->sub)
     freedir_rec(dr->sub);
- 
+
   /* update references */
   if(dr->parent && dr->parent->sub == dr)
     dr->parent->sub = dr->next;
@@ -230,13 +231,7 @@ void freedir(struct dir *dr) {
 
   /* update sizes of parent directories if this isn't a hard link.
    * If this is a hard link, freedir_hlnk() would have done so already */
-  for(tmp=dr->parent; tmp; tmp=tmp->parent) {
-    if(!(dr->flags & FF_HLNKC)) {
-      tmp->size -= dr->size;
-      tmp->asize -= dr->asize;
-    }
-    tmp->items -= dr->items+1;
-  }
+  addparentstats(dr->parent, dr->flags & FF_HLNKC ? 0 : -dr->size, dr->flags & FF_HLNKC ? 0 : -dr->asize, -(dr->items+1));
 
   free(dr);
 }
@@ -280,3 +275,19 @@ char *getpath(struct dir *cur) {
   return dat;
 }
 
+
+struct dir *getroot(struct dir *d) {
+  while(d && d->parent)
+    d = d->parent;
+  return d;
+}
+
+
+void addparentstats(struct dir *d, int64_t size, int64_t asize, int items) {
+  while(d) {
+    d->size += size;
+    d->asize += asize;
+    d->items += items;
+    d = d->parent;
+  }
+}
