@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2011 Yoran Heling
+  Copyright (c) 2007-2012 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -52,7 +52,7 @@ extern int subwinr, subwinc;
 int ncresize(int, int);
 
 /* creates a new centered window with border */
-void nccreate(int, int, char *);
+void nccreate(int, int, const char *);
 
 /* printf something somewhere in the last created window */
 void ncprint(int, int, char *, ...);
@@ -66,10 +66,10 @@ void ncprint(int, int, char *, ...);
 char *cropstr(const char *, int);
 
 /* formats size in the form of xxx.xXB */
-char *formatsize(const off_t);
+char *formatsize(int64_t );
 
 /* int2string with thousand separators */
-char *fullsize(const off_t);
+char *fullsize(int64_t);
 
 /* recursively free()s a directory tree */
 void freedir(struct dir *);
@@ -78,5 +78,32 @@ void freedir(struct dir *);
    returned pointer will be overwritten with a subsequent call */
 char *getpath(struct dir *);
 
+/* returns the root element of the given dir struct */
+struct dir *getroot(struct dir *);
+
+/* Adds a value to the size, asize and items fields of *d and its parents */
+void addparentstats(struct dir *, int64_t, int64_t, int);
+
+
+/* A simple stack implemented in macros */
+#define nstack_init(_s) do {\
+    (_s)->size = 10;\
+    (_s)->top = 0;\
+    (_s)->list = malloc(10*sizeof(*(_s)->list));\
+  } while(0)
+
+#define nstack_push(_s, _v) do {\
+    if((_s)->size <= (_s)->top) {\
+      (_s)->size *= 2;\
+      (_s)->list = realloc((_s)->list, (_s)->size*sizeof(*(_s)->list));\
+    }\
+    (_s)->list[(_s)->top++] = _v;\
+  } while(0)
+
+#define nstack_pop(_s) (_s)->top--
+#define nstack_top(_s, _d) ((_s)->top > 0 ? (_s)->list[(_s)->top-1] : (_d))
+#define nstack_free(_s) free((_s)->list)
+
+
 #endif
 
